create table SYS_SERVER (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    NAME varchar(190),
    IS_RUNNING boolean,
    DATA text,
    primary key (ID)
)^

create unique index IDX_SYS_SERVER_UNIQ_NAME on SYS_SERVER (NAME)^

/***************************************************************************************************/

create table SYS_CONFIG (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    --
    NAME varchar(190) not null, -- See PL-8935, 190 length is set due to MySQL problem with encoding
    VALUE_ text not null,
    --
    primary key (ID)
)^

create unique index IDX_SYS_CONFIG_UNIQ_NAME on SYS_CONFIG (NAME)^

/**********************************************************************************************/

create table SYS_FILE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    NAME varchar(500) not null,
    EXT varchar(20),
    FILE_SIZE bigint,
    CREATE_DATE datetime(3),
    --
    primary key (ID)
)^

/**********************************************************************************************/

create table SYS_LOCK_CONFIG (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    --
    NAME varchar(100),
    TIMEOUT_SEC integer,
    --
    primary key (ID)
)^

/**********************************************************************************************/

create table SYS_ENTITY_STATISTICS (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    --
    NAME varchar(50),
    INSTANCE_COUNT bigint,
    FETCH_UI integer,
    MAX_FETCH_UI integer,
    LAZY_COLLECTION_THRESHOLD integer,
    LOOKUP_SCREEN_THRESHOLD integer,
    --
    primary key (ID)
)^

create unique index IDX_SYS_ENTITY_STATISTICS_UNIQ_NAME on SYS_ENTITY_STATISTICS (NAME)^

/**********************************************************************************************/

create table SYS_SCHEDULED_TASK (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    DEFINED_BY varchar(1) default 'B',
    CLASS_NAME varchar(500),
    SCRIPT_NAME varchar(500),
    BEAN_NAME varchar(50),
    METHOD_NAME varchar(50),
    METHOD_PARAMS varchar(4000),
    USER_NAME varchar(50),
    IS_SINGLETON boolean,
    IS_ACTIVE boolean,
    PERIOD_ integer,
    TIMEOUT integer,
    START_DATE datetime(3),
    TIME_FRAME integer,
    START_DELAY integer,
    PERMITTED_SERVERS varchar(4096),
    LOG_START boolean,
    LOG_FINISH boolean,
    LAST_START_TIME datetime(3),
    LAST_START_SERVER varchar(512),
    DESCRIPTION varchar(1000),
    CRON varchar(100),
    SCHEDULING_TYPE varchar(1) default 'P',
    --
    primary key (ID)
)^

-- create unique index IDX_SYS_SCHEDULED_TASK_UNIQ_BEAN_METHOD on SYS_SCHEDULED_TASK (BEAN_NAME, METHOD_NAME, METHOD_PARAMS, DELETE_TS)^

/**********************************************************************************************/

create table SYS_SCHEDULED_EXECUTION (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    TASK_ID varchar(32),
    SERVER varchar(512),
    START_TIME datetime(3),
    FINISH_TIME datetime(3),
    RESULT text,
    --
    primary key (ID),
    constraint SYS_SCHEDULED_EXECUTION_TASK foreign key (TASK_ID) references SYS_SCHEDULED_TASK(ID)
)^

create index IDX_SYS_SCHEDULED_EXECUTION_TASK_START_TIME  on SYS_SCHEDULED_EXECUTION (TASK_ID, START_TIME)^
create index IDX_SYS_SCHEDULED_EXECUTION_TASK_FINISH_TIME on SYS_SCHEDULED_EXECUTION (TASK_ID, FINISH_TIME)^

/**********************************************************************************************/

create table SEC_ROLE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    SYS_TENANT_ID varchar(255),
    SYS_TENANT_ID_NN varchar(255),
    --
    NAME varchar(190) not null,
    LOC_NAME varchar(255),
    DESCRIPTION varchar(1000),
    IS_DEFAULT_ROLE boolean,
    ROLE_TYPE integer,
    SECURITY_SCOPE varchar(255),
    --
    primary key (ID)
)^

create unique index IDX_SEC_ROLE_UNIQ_NAME on SEC_ROLE (NAME, SYS_TENANT_ID_NN, DELETE_TS_NN)^

create trigger SEC_ROLE_SYS_TENANT_ID_NN_INSERT_TRIGGER before insert on SEC_ROLE
for each row set NEW.SYS_TENANT_ID_NN = if (NEW.SYS_TENANT_ID is null, 'no_tenant', NEW.SYS_TENANT_ID)^

create trigger SEC_ROLE_SYS_TENANT_ID_NN_AND_DELETE_TS_NN_UPDATE_TRIGGER before update on SEC_ROLE
for each row
begin
    if not(NEW.SYS_TENANT_ID <=> OLD.SYS_TENANT_ID) then
        set NEW.SYS_TENANT_ID_NN = NEW.SYS_TENANT_ID;
    end if;
	if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
		set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
	end if;
end^

/**********************************************************************************************/

create table SEC_GROUP (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    SYS_TENANT_ID varchar(255),
    SYS_TENANT_ID_NN varchar(255),
    --
    NAME varchar(190) not null,
    PARENT_ID varchar(32),
    --
    primary key (ID),
    constraint SEC_GROUP_PARENT foreign key (PARENT_ID) references SEC_GROUP(ID)
)^

create unique index IDX_SEC_GROUP_UNIQ_NAME on SEC_GROUP (NAME, SYS_TENANT_ID_NN, DELETE_TS_NN)^

create trigger SEC_GROUP_SYS_TENANT_ID_NN_INSERT_TRIGGER before insert on SEC_GROUP
for each row set NEW.SYS_TENANT_ID_NN = if (NEW.SYS_TENANT_ID is null, 'no_tenant', NEW.SYS_TENANT_ID)^

create trigger SEC_GROUP_SYS_TENANT_ID_NN_AND_DELETE_TS_NN_UPDATE_TRIGGER before update on SEC_GROUP
for each row
begin
    if not(NEW.SYS_TENANT_ID <=> OLD.SYS_TENANT_ID) then
		set NEW.SYS_TENANT_ID_NN = NEW.SYS_TENANT_ID;
	end if;
	if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
		set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
	end if;
end^
/**********************************************************************************************/

create table SEC_GROUP_HIERARCHY (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    GROUP_ID varchar(32),
    PARENT_ID varchar(32),
    HIERARCHY_LEVEL integer,
    --
    primary key (ID),
    constraint SEC_GROUP_HIERARCHY_GROUP foreign key (GROUP_ID) references SEC_GROUP(ID),
    constraint SEC_GROUP_HIERARCHY_PARENT foreign key (PARENT_ID) references SEC_GROUP(ID)
)^

/**********************************************************************************************/

create table SEC_USER (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    SYS_TENANT_ID varchar(255),
    SYS_TENANT_ID_NN varchar(255),
    --
    LOGIN varchar(50) not null,
    LOGIN_LC varchar(50) not null,
    PASSWORD varchar(255),
    PASSWORD_ENCRYPTION varchar(50),
    NAME varchar(255),
    FIRST_NAME varchar(255),
    LAST_NAME varchar(255),
    MIDDLE_NAME varchar(255),
    POSITION_ varchar(255),
    EMAIL varchar(100),
    LANGUAGE_ varchar(20),
    TIME_ZONE varchar(50),
    TIME_ZONE_AUTO boolean,
    ACTIVE boolean,
    GROUP_ID varchar(32),
    GROUP_NAMES varchar(255),
    IP_MASK varchar(200),
    CHANGE_PASSWORD_AT_LOGON boolean,
    --
    primary key (ID),
    constraint SEC_USER_GROUP foreign key (GROUP_ID) references SEC_GROUP(ID)
)^

create unique index IDX_SEC_USER_UNIQ_LOGIN on SEC_USER (LOGIN_LC, SYS_TENANT_ID_NN, DELETE_TS_NN)^

create trigger SEC_USER_SYS_TENANT_ID_NN_INSERT_TRIGGER before insert on SEC_USER
for each row set NEW.SYS_TENANT_ID_NN = if (NEW.SYS_TENANT_ID is null, 'no_tenant', NEW.SYS_TENANT_ID)^

create trigger SEC_USER_SYS_TENANT_ID_NN_AND_DELETE_TS_NN_UPDATE_TRIGGER before update on SEC_USER
for each row
begin
    if not(NEW.SYS_TENANT_ID <=> OLD.SYS_TENANT_ID) then
    	set NEW.SYS_TENANT_ID_NN = NEW.SYS_TENANT_ID;
	end if;
	if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
		set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
	end if;
end^

/**********************************************************************************************/

create table SEC_USER_ROLE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    --
    USER_ID varchar(32),
    ROLE_ID varchar(32),
    ROLE_NAME varchar(50),
    --
    primary key (ID),
    constraint SEC_USER_ROLE_PROFILE foreign key (USER_ID) references SEC_USER(ID),
    constraint SEC_USER_ROLE_ROLE foreign key (ROLE_ID) references SEC_ROLE(ID)
)^

create unique index IDX_SEC_USER_ROLE_UNIQ_ROLE on SEC_USER_ROLE (USER_ID, ROLE_ID, DELETE_TS_NN)^

create trigger SEC_USER_ROLE_DELETE_TS_NN_TRIGGER before update on SEC_USER_ROLE
for each row
	if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
		set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
	end if^



/**********************************************************************************************/

create table SEC_PERMISSION (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    --
    PERMISSION_TYPE integer,
    TARGET varchar(100),
    VALUE_ integer,
    ROLE_ID varchar(32),
    --
    primary key (ID),
    constraint SEC_PERMISSION_ROLE foreign key (ROLE_ID) references SEC_ROLE(ID)
)^

create unique index IDX_SEC_PERMISSION_UNIQUE on SEC_PERMISSION (ROLE_ID, PERMISSION_TYPE, TARGET, DELETE_TS_NN)^

create trigger SEC_PERMISSION_DELETE_TS_NN_TRIGGER before update on SEC_PERMISSION
for each row
	if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
		set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
	end if^


/**********************************************************************************************/

create table SEC_CONSTRAINT (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    CODE varchar(255),
    CHECK_TYPE varchar(50) default 'db',
    OPERATION_TYPE varchar(50) default 'read',
    ENTITY_NAME varchar(255) not null,
    JOIN_CLAUSE varchar(500),
    WHERE_CLAUSE varchar(1000),
    GROOVY_SCRIPT text,
    FILTER_XML text,
    IS_ACTIVE boolean default true,
    GROUP_ID varchar(32),
    --
    primary key (ID),
    constraint SEC_CONSTRAINT_GROUP foreign key (GROUP_ID) references SEC_GROUP(ID)
)^

/**********************************************************************************************/

create table SEC_LOCALIZED_CONSTRAINT_MSG (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    --
    ENTITY_NAME varchar(150) not null,
    OPERATION_TYPE varchar(40) not null,
    VALUES_ text,
    --
    primary key (ID)
)^

create unique index IDX_SEC_LOC_CNSTRNT_MSG_UNIQUE
  on SEC_LOCALIZED_CONSTRAINT_MSG (ENTITY_NAME, OPERATION_TYPE, DELETE_TS_NN)^

create trigger SEC_LOCALIZED_CONSTRAINT_MSG_DELETE_TS_NN_TRIGGER before update on SEC_LOCALIZED_CONSTRAINT_MSG
for each row
  if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
    set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
  end if^

/**********************************************************************************************/

create table SEC_SESSION_ATTR (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    NAME varchar(50),
    STR_VALUE varchar(1000),
    DATATYPE varchar(20),
    GROUP_ID varchar(32),
    --
    primary key (ID),
    constraint SEC_SESSION_ATTR_GROUP foreign key (GROUP_ID) references SEC_GROUP(ID)
)^

/**********************************************************************************************/

create table SEC_USER_SETTING (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    --
    USER_ID varchar(32),
    CLIENT_TYPE char(1),
    NAME varchar(190),
    VALUE_ text,
    --
    primary key (ID),
    constraint SEC_USER_SETTING_USER foreign key (USER_ID) references SEC_USER(ID),
    constraint SEC_USER_SETTING_UNIQ unique (USER_ID, NAME, CLIENT_TYPE)
)^

/**********************************************************************************************/

create table SEC_USER_SUBSTITUTION (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    USER_ID varchar(32) not null,
    SUBSTITUTED_USER_ID varchar(32) not null,
    START_DATE datetime(3),
    END_DATE datetime(3),
    --
    primary key (ID),
    constraint FK_SEC_USER_SUBSTITUTION_USER foreign key (USER_ID) references SEC_USER(ID),
    constraint FK_SEC_USER_SUBSTITUTION_SUBSTITUTED_USER foreign key (SUBSTITUTED_USER_ID) references SEC_USER(ID)
)^

/**********************************************************************************************/

create table SEC_LOGGED_ENTITY (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    --
    NAME varchar(100),
    AUTO boolean,
    MANUAL boolean,
    --
    primary key (ID),
    constraint SEC_LOGGED_ENTITY_UNIQ_NAME unique (NAME)
)^

/**********************************************************************************************/

create table SEC_LOGGED_ATTR (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    --
    ENTITY_ID varchar(32),
    NAME varchar(50),
    --
    primary key (ID),
    constraint FK_SEC_LOGGED_ATTR_ENTITY foreign key (ENTITY_ID) references SEC_LOGGED_ENTITY(ID),
    constraint SEC_LOGGED_ATTR_UNIQ_NAME unique (ENTITY_ID, NAME)
)^

/**********************************************************************************************/

create table SEC_ENTITY_LOG (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    EVENT_TS datetime(3),
    USER_ID varchar(32),
    CHANGE_TYPE char(1),
    ENTITY varchar(100),
    ENTITY_INSTANCE_NAME varchar(1000),
    ENTITY_ID varchar(32),
    STRING_ENTITY_ID varchar(190),
    INT_ENTITY_ID integer,
    LONG_ENTITY_ID bigint,
    CHANGES text,
    --
    primary key (ID),
    constraint FK_SEC_ENTITY_LOG_USER foreign key (USER_ID) references SEC_USER(ID)
)^

create index IDX_SEC_ENTITY_LOG_ENTITY_ID on SEC_ENTITY_LOG (ENTITY_ID)^
create index IDX_SEC_ENTITY_LOG_SENTITY_ID on SEC_ENTITY_LOG (STRING_ENTITY_ID)^
create index IDX_SEC_ENTITY_LOG_IENTITY_ID on SEC_ENTITY_LOG (INT_ENTITY_ID)^
create index IDX_SEC_ENTITY_LOG_LENTITY_ID on SEC_ENTITY_LOG (LONG_ENTITY_ID)^

/**********************************************************************************************/

create table SEC_FILTER (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    COMPONENT varchar(200),
    NAME varchar(255),
    CODE varchar(200),
    XML text,
    USER_ID varchar(32),
    GLOBAL_DEFAULT boolean,
    --
    primary key (ID),
    constraint FK_SEC_FILTER_USER foreign key (USER_ID) references SEC_USER(ID)
)^

/**********************************************************************************************/

create table SYS_FOLDER (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    FOLDER_TYPE char(1),
    PARENT_ID varchar(32),
    NAME varchar(100),
    TAB_NAME varchar(100),
    SORT_ORDER integer,
    --
    primary key (ID),
    constraint FK_SYS_FOLDER_PARENT foreign key (PARENT_ID) references SYS_FOLDER(ID)
)^

/**********************************************************************************************/

create table SYS_APP_FOLDER (
    FOLDER_ID varchar(32),
    FILTER_COMPONENT varchar(200),
    FILTER_XML varchar(7000),
    VISIBILITY_SCRIPT text,
    QUANTITY_SCRIPT text,
    APPLY_DEFAULT boolean,
    --
    primary key (FOLDER_ID),
    constraint FK_SYS_APP_FOLDER_FOLDER foreign key (FOLDER_ID) references SYS_FOLDER(ID)
)^

/**********************************************************************************************/

create table SEC_PRESENTATION (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    COMPONENT varchar(200),
    NAME varchar(255),
    XML varchar(7000),
    USER_ID varchar(32),
    IS_AUTO_SAVE boolean,
    --
    primary key (ID),
    constraint SEC_PRESENTATION_USER foreign key (USER_ID) references SEC_USER(ID)
)^

/**********************************************************************************************/

create table SEC_SEARCH_FOLDER (
    FOLDER_ID varchar(32),
    FILTER_COMPONENT varchar(200),
    FILTER_XML varchar(7000),
    USER_ID varchar(32),
    PRESENTATION_ID varchar(32),
    APPLY_DEFAULT boolean,
    IS_SET boolean,
    ENTITY_TYPE varchar(50),
    --
    primary key (FOLDER_ID),
    constraint FK_SEC_SEARCH_FOLDER_FOLDER foreign key (FOLDER_ID) references SYS_FOLDER(ID),
    constraint FK_SEC_SEARCH_FOLDER_USER foreign key (USER_ID) references SEC_USER(ID),
    constraint FK_SEC_SEARCH_FOLDER_PRESENTATION foreign key (PRESENTATION_ID)
        references SEC_PRESENTATION(ID)
        on delete set null
)^

create index IDX_SEC_SEARCH_FOLDER_USER on SEC_SEARCH_FOLDER (USER_ID)^

/**********************************************************************************************/

create table SYS_FTS_QUEUE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    --
    ENTITY_ID varchar(32),
    STRING_ENTITY_ID varchar(190),
    INT_ENTITY_ID integer,
    LONG_ENTITY_ID bigint,
    ENTITY_NAME varchar(200),
    CHANGE_TYPE char(1),
    SOURCE_HOST varchar(255),
    INDEXING_HOST varchar(190),
    FAKE boolean,
    --
    primary key (ID)
)^

create index IDX_SYS_FTS_QUEUE_IDXHOST_CRTS on SYS_FTS_QUEUE (INDEXING_HOST, CREATE_TS)^

/**********************************************************************************************/

create table SEC_SCREEN_HISTORY (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    USER_ID varchar(32),
    CAPTION varchar(255),
    URL text,
    ENTITY_ID varchar(32),
    STRING_ENTITY_ID varchar(190),
    INT_ENTITY_ID integer,
    LONG_ENTITY_ID bigint,
    SUBSTITUTED_USER_ID varchar(32),
      --
    primary key (ID),
    constraint FK_SEC_HISTORY_USER foreign key (USER_ID) references SEC_USER (ID),
    constraint FK_SEC_HISTORY_SUBSTITUTED_USER foreign key (SUBSTITUTED_USER_ID) references SEC_USER (ID)
)^

create index IDX_SEC_SCREEN_HISTORY_USER on SEC_SCREEN_HISTORY (USER_ID)^
create index IDX_SEC_SCREEN_HIST_SUB_USER on SEC_SCREEN_HISTORY (SUBSTITUTED_USER_ID)^
create index IDX_SEC_SCREEN_HISTORY_ENTITY_ID on SEC_SCREEN_HISTORY (ENTITY_ID)^
create index IDX_SEC_SCREEN_HISTORY_SENTITY_ID on SEC_SCREEN_HISTORY (STRING_ENTITY_ID)^
create index IDX_SEC_SCREEN_HISTORY_IENTITY_ID on SEC_SCREEN_HISTORY (INT_ENTITY_ID)^
create index IDX_SEC_SCREEN_HISTORY_LENTITY_ID on SEC_SCREEN_HISTORY (LONG_ENTITY_ID)^

/**********************************************************************************************/

create table SYS_SENDING_MESSAGE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    ADDRESS_TO text,
    ADDRESS_CC text,
    ADDRESS_BCC text,
    ADDRESS_FROM varchar(100),
    CAPTION varchar(500),
    EMAIL_HEADERS varchar(500),
    CONTENT_TEXT text,
    CONTENT_TEXT_FILE_ID varchar(32),
    DEADLINE datetime(3),
    STATUS int,
    DATE_SENT datetime(3),
    ATTEMPTS_COUNT int,
    ATTEMPTS_MADE int,
    ATTACHMENTS_NAME text,
    BODY_CONTENT_TYPE varchar(50),
    --
    primary key (ID),
    constraint FK_SYS_SENDING_MESSAGE_CONTENT_FILE foreign key (CONTENT_TEXT_FILE_ID) references SYS_FILE(ID)
)^

create index IDX_SYS_SENDING_MESSAGE_STATUS on SYS_SENDING_MESSAGE (STATUS)^

create index IDX_SYS_SENDING_MESSAGE_DATE_SENT on SYS_SENDING_MESSAGE (DATE_SENT)^

create index IDX_SYS_SENDING_MESSAGE_UPDATE_TS on SYS_SENDING_MESSAGE (UPDATE_TS)^

 #------------------------------------------------------------------------------------------------------------

create table SYS_SENDING_ATTACHMENT (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    MESSAGE_ID varchar(32),
    CONTENT longblob,
    CONTENT_FILE_ID varchar(32),
    CONTENT_ID varchar(50),
    NAME varchar(500),
    DISPOSITION varchar(50),
    TEXT_ENCODING varchar(50),

    --
    primary key (ID),
    constraint FK_SYS_SENDING_ATTACHMENT_SENDING_MESSAGE foreign key (MESSAGE_ID) references SYS_SENDING_MESSAGE (ID),
    constraint FK_SYS_SENDING_ATTACHMENT_CONTENT_FILE foreign key (CONTENT_FILE_ID) references SYS_FILE (ID)
)^

create index SYS_SENDING_ATTACHMENT_MESSAGE_IDX on SYS_SENDING_ATTACHMENT (MESSAGE_ID)^

/**********************************************************************************************/

create table SYS_ENTITY_SNAPSHOT (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    ENTITY_META_CLASS varchar(50) not null,
    ENTITY_ID varchar(32),
    STRING_ENTITY_ID varchar(190),
    INT_ENTITY_ID integer,
    LONG_ENTITY_ID bigint,
    AUTHOR_ID varchar(32) not null,
    VIEW_XML text not null,
    SNAPSHOT_XML text not null,
    SNAPSHOT_DATE datetime(3) not null,
    --
    primary key (ID),
    constraint FK_SYS_ENTITY_SNAPSHOT_AUTHOR_ID foreign key (AUTHOR_ID) references SEC_USER(ID)
)^

create index IDX_SYS_ENTITY_SNAPSHOT_ENTITY_ID on SYS_ENTITY_SNAPSHOT (ENTITY_ID)^
create index IDX_SYS_ENTITY_SNAPSHOT_SENTITY_ID on SYS_ENTITY_SNAPSHOT (STRING_ENTITY_ID)^
create index IDX_SYS_ENTITY_SNAPSHOT_IENTITY_ID on SYS_ENTITY_SNAPSHOT (INT_ENTITY_ID)^
create index IDX_SYS_ENTITY_SNAPSHOT_LENTITY_ID on SYS_ENTITY_SNAPSHOT (LONG_ENTITY_ID)^

/**********************************************************************************************/

create table SYS_CATEGORY(
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    DELETE_TS_NN datetime(3) not null default '1000-01-01 00:00:00.000',
    --
    NAME varchar(190) not null,
    SPECIAL varchar(50),
    ENTITY_TYPE varchar(100) not null,
    IS_DEFAULT boolean,
    DISCRIMINATOR integer,
    LOCALE_NAMES varchar(1000),
    --
    primary key (ID)
)^

create unique index IDX_SYS_CATEGORY_UNIQ_NAME_ENTITY_TYPE on SYS_CATEGORY (NAME, ENTITY_TYPE, DELETE_TS_NN)^

create trigger SYS_CATEGORY_DELETE_TS_NN_TRIGGER before update on SYS_CATEGORY
for each row
	if not(NEW.DELETE_TS <=> OLD.DELETE_TS) then
		set NEW.DELETE_TS_NN = if (NEW.DELETE_TS is null, '1000-01-01 00:00:00.000', NEW.DELETE_TS);
	end if^

/**********************************************************************************************/

create table SYS_CATEGORY_ATTR (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    --
    CATEGORY_ENTITY_TYPE text,
    NAME varchar(255),
    CODE varchar(100) not null,
    DESCRIPTION varchar(1000),
    CATEGORY_ID varchar(32) not null,
    ENTITY_CLASS varchar(500),
    DATA_TYPE varchar(200),
    DEFAULT_STRING text,
    DEFAULT_INT integer,
    DEFAULT_DOUBLE numeric(36,6),
    DEFAULT_DECIMAL numeric(36,10),
    DEFAULT_DATE datetime(3),
    DEFAULT_DATE_WO_TIME date,
    DEFAULT_DATE_IS_CURRENT boolean,
    DEFAULT_BOOLEAN boolean,
    DEFAULT_ENTITY_VALUE varchar(36),
    DEFAULT_STR_ENTITY_VALUE varchar(255),
    DEFAULT_INT_ENTITY_VALUE integer,
    DEFAULT_LONG_ENTITY_VALUE bigint,
    ENUMERATION varchar(500),
    ORDER_NO integer,
    SCREEN varchar(255),
    REQUIRED boolean,
    LOOKUP boolean,
    TARGET_SCREENS text,
    WIDTH varchar(20),
    ROWS_COUNT integer,
    IS_COLLECTION boolean,
    JOIN_CLAUSE text,
    WHERE_CLAUSE text,
    FILTER_XML longtext,
    LOCALE_NAMES varchar(1000),
    ENUMERATION_LOCALES text,
    LOCALE_DESCRIPTIONS text,
    ATTRIBUTE_CONFIGURATION_JSON text,
    --
    primary key (ID),
    constraint SYS_CATEGORY_ATTR_CATEGORY_ID foreign key (CATEGORY_ID) references SYS_CATEGORY(ID)
)^

/**********************************************************************************************/

create table SYS_ATTR_VALUE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    --
    CATEGORY_ATTR_ID varchar(32) not null,
    ENTITY_ID varchar(32),
    STRING_ENTITY_ID varchar(190),
    INT_ENTITY_ID integer,
    LONG_ENTITY_ID bigint,
    STRING_VALUE text,
    INTEGER_VALUE integer,
    DOUBLE_VALUE numeric(36,6),
    DECIMAL_VALUE numeric(36,10),
    DATE_VALUE datetime(3),
    DATE_WO_TIME_VALUE date,
    BOOLEAN_VALUE boolean,
    ENTITY_VALUE varchar(36),
    STRING_ENTITY_VALUE varchar(190),
    INT_ENTITY_VALUE integer,
    LONG_ENTITY_VALUE bigint,
    CODE varchar(100) not null,
    PARENT_ID varchar(32),
    --
    primary key (ID),
    constraint SYS_ATTR_VALUE_CATEGORY_ATTR_ID foreign key (CATEGORY_ATTR_ID) references SYS_CATEGORY_ATTR(ID),
    constraint SYS_ATTR_VALUE_ATTR_VALUE_PARENT_ID foreign key (PARENT_ID) references SYS_ATTR_VALUE(ID)

)^

create index IDX_SYS_ATTR_VALUE_ENTITY on SYS_ATTR_VALUE(ENTITY_ID)^
create index IDX_SYS_ATTR_VALUE_SENTITY on SYS_ATTR_VALUE(STRING_ENTITY_ID)^
create index IDX_SYS_ATTR_VALUE_IENTITY on SYS_ATTR_VALUE(INT_ENTITY_ID)^
create index IDX_SYS_ATTR_VALUE_LENTITY on SYS_ATTR_VALUE(LONG_ENTITY_ID)^

/**********************************************************************************************/

create table SYS_JMX_INSTANCE (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    --
    NODE_NAME varchar(255),
    ADDRESS varchar(500) not null,
    LOGIN varchar(50) not null,
    PASSWORD varchar(255) not null,
    --
    primary key (ID)
)^

/**********************************************************************************************/

create table SYS_QUERY_RESULT (
    ID bigint auto_increment not null,
    SESSION_ID varchar(32) not null,
    QUERY_KEY integer not null,
    ENTITY_ID varchar(32),
    STRING_ENTITY_ID varchar(190),
    INT_ENTITY_ID integer,
    LONG_ENTITY_ID bigint,
    --
    primary key (ID)
)^

create index IDX_SYS_QUERY_RESULT_ENTITY_SESSION_KEY on SYS_QUERY_RESULT (ENTITY_ID, SESSION_ID, QUERY_KEY)^
create index IDX_SYS_QUERY_RESULT_SENTITY_SESSION_KEY on SYS_QUERY_RESULT (STRING_ENTITY_ID, SESSION_ID, QUERY_KEY)^
create index IDX_SYS_QUERY_RESULT_IENTITY_SESSION_KEY on SYS_QUERY_RESULT (INT_ENTITY_ID, SESSION_ID, QUERY_KEY)^
create index IDX_SYS_QUERY_RESULT_LENTITY_SESSION_KEY on SYS_QUERY_RESULT (LONG_ENTITY_ID, SESSION_ID, QUERY_KEY)^

create index IDX_SYS_QUERY_RESULT_SESSION_KEY on SYS_QUERY_RESULT (SESSION_ID, QUERY_KEY)^

/**********************************************************************************************/

create table SEC_REMEMBER_ME (
    ID varchar(32),
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    VERSION integer not null default 1,
    --
    USER_ID varchar(32) not null,
    TOKEN varchar(32) not null,
    --
    primary key (ID),
    constraint FK_SEC_REMEMBER_ME_USER foreign key (USER_ID) references SEC_USER(ID)
)^
create index IDX_SEC_REMEMBER_ME_USER on SEC_REMEMBER_ME(USER_ID)^
create index IDX_SEC_REMEMBER_ME_TOKEN on SEC_REMEMBER_ME(TOKEN)^

/**********************************************************************************************/

create table SEC_SESSION_LOG (
    ID varchar(32) not null,
    VERSION integer not null default 1,
    CREATE_TS datetime(3),
    CREATED_BY varchar(50),
    UPDATE_TS datetime(3),
    UPDATED_BY varchar(50),
    DELETE_TS datetime(3),
    DELETED_BY varchar(50),
    SYS_TENANT_ID varchar(255),
    --
    SESSION_ID varchar(32) not null,
    USER_ID varchar(32) not null,
    SUBSTITUTED_USER_ID varchar(32),
    USER_DATA text,
    LAST_ACTION integer not null,
    CLIENT_INFO varchar(512),
    CLIENT_TYPE varchar(10),
    ADDRESS varchar(255),
    STARTED_TS datetime(3),
    FINISHED_TS datetime(3),
    SERVER_ID varchar(128),
    --
    primary key (ID),
    constraint FK_SESSION_LOG_ENTRY_USER foreign key (USER_ID) references SEC_USER(ID),
    constraint FK_SESSION_LOG_ENTRY_SUBUSER foreign key (SUBSTITUTED_USER_ID) references SEC_USER(ID)
)^

create index IDX_SESSION_LOG_ENTRY_USER on SEC_SESSION_LOG (USER_ID)^
create index IDX_SESSION_LOG_ENTRY_SUBUSER on SEC_SESSION_LOG (SUBSTITUTED_USER_ID)^
create index IDX_SESSION_LOG_ENTRY_SESSION on SEC_SESSION_LOG (SESSION_ID)^
create index IDX_SESSION_LOG_STARTED_TS on SEC_SESSION_LOG (STARTED_TS DESC)^

/**********************************************************************************************/

drop function if exists newid^
create function newid() returns varchar(32) not deterministic no sql
return replace(uuid(), '-', '')^

/**********************************************************************************************/

insert into SEC_GROUP (ID, CREATE_TS, VERSION, NAME, PARENT_ID)
values ('0fa2b1a51d684d699fbddff348347f93', current_timestamp, 0, 'Company', null)^

insert into SEC_USER (ID, CREATE_TS, VERSION, LOGIN, LOGIN_LC, PASSWORD, PASSWORD_ENCRYPTION, NAME, GROUP_ID, ACTIVE)
values ('608859871b61424794c7dff348347f93', current_timestamp, 0, 'admin', 'admin',
'$2a$10$vQx8b8B7jzZ0rQmtuK4YDOKp7nkmUCFjPx6DMT.voPtetNHFOsaOu', 'bcrypt',
'Administrator', '0fa2b1a51d684d699fbddff348347f93', 1)^

insert into SEC_USER (ID, CREATE_TS, VERSION, LOGIN, LOGIN_LC, PASSWORD, NAME, GROUP_ID, ACTIVE)
values ('a405db59e6744f638afe269dda788fe8', now(), 0, 'anonymous', 'anonymous', null,
'Anonymous', '0fa2b1a51d684d699fbddff348347f93', true)^

insert into SEC_USER_ROLE (ID, CREATE_TS, VERSION, USER_ID, ROLE_NAME)
values ('6736effb9dfc4430973a69868606b09c', current_timestamp, 0, '608859871b61424794c7dff348347f93', 'system-full-access')^

insert into SEC_FILTER (ID,CREATE_TS,CREATED_BY,VERSION,UPDATE_TS,UPDATED_BY,DELETE_TS,DELETED_BY,COMPONENT,NAME,XML,USER_ID,GLOBAL_DEFAULT)
values ('b61d18cbe79a46f3b16deaf4aebb10dd',{ts '2010-03-01 11:14:06.830'},'admin',2,{ts '2010-03-01 11:52:53.170'},'admin',null,null,'[sec$User.browse].genericFilter','Search by role',
'<?xml version="1.0" encoding="UTF-8"?>
<filter>
  <and>
    <c name="UrMxpkfMGn" class="com.haulmont.cuba.security.entity.Role" type="CUSTOM" locCaption="Role" entityAlias="u" join="join u.userRoles ur">ur.role.id = :component$genericFilter.UrMxpkfMGn32565
      <param name="component$genericFilter.UrMxpkfMGn32565">NULL</param>
    </c>
  </and>
</filter>',
'608859871b61424794c7dff348347f93',false)^

/**********************************************************************************************/

create table IF NOT EXISTS SYS_SEQUENCE (
    NAME varchar(100) not null,
    INCREMENT int unsigned not null default 1,
    CURR_VALUE bigint unsigned default 0,
    primary key (NAME)
)^

create unique index IDX_SYS_SEQUENCE_UNIQUE_NAME on SYS_SEQUENCE (NAME)^