/*
 * Copyright (c) 2008-2016 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.haulmont.chile.core.datatypes.impl;

import com.haulmont.chile.core.annotations.JavaClass;
import com.haulmont.chile.core.datatypes.Datatype;
import com.haulmont.chile.core.datatypes.FormatStrings;
import com.haulmont.chile.core.datatypes.FormatStringsRegistry;
import com.haulmont.cuba.core.global.AppBeans;
import org.apache.commons.lang3.StringUtils;

import javax.annotation.Nullable;
import java.text.ParseException;
import java.util.Locale;

@JavaClass(Boolean.class)
public class BooleanDatatype implements Datatype<Boolean> {

    @Override
    public String format(@Nullable Object value) {
        return value == null ? "" : Boolean.toString((Boolean) value);
    }

    @Override
    public String format(@Nullable Object value, Locale locale) {
        if (value == null) {
            return "";
        }

        FormatStrings formatStrings = AppBeans.get(FormatStringsRegistry.class).getFormatStrings(locale);
        if (formatStrings == null) {
            return format(value);
        }

        return (boolean) value ? formatStrings.getTrueString() : formatStrings.getFalseString();
    }

    @Nullable
    protected Boolean parse(@Nullable String value, String trueString, String falseString) throws ParseException {
        if (StringUtils.isBlank(value)) {
            return null;
        }
        if (trueString.equalsIgnoreCase(value)) {
            return Boolean.TRUE;
        }
        if (falseString.equalsIgnoreCase(value)) {
            return Boolean.FALSE;
        }

        throw new ParseException(String.format("Can't parse '%s'", value), 0);
    }

    @Override
    public Boolean parse(@Nullable String value) throws ParseException {
        return parse(value, "true", "false");
    }

    @Override
    public Boolean parse(@Nullable String value, Locale locale) throws ParseException {
        if (StringUtils.isBlank(value)) {
            return null;
        }

        FormatStrings formatStrings = AppBeans.get(FormatStringsRegistry.class).getFormatStrings(locale);
        if (formatStrings == null) {
            return parse(value);
        }

        return parse(value, formatStrings.getTrueString(), formatStrings.getFalseString());
    }

    @Override
    public String toString() {
        return getClass().getSimpleName();
    }

    @Deprecated
    public final static String NAME = "boolean";
}